#!/bin/bash
#
# Support program for VMware Remote Console.
#
# Collects various configuration and log files
# for use when troubleshooting VMware Remote Console.
#

# Print how to use this executable, then terminate with an error.
usage() {
   banner \
"Usage: $0" \
"" \
"   -a" \
"      Runs program in automatic mode" \
"      Output the ID of this process." \
"      Output the path of the generated file." \
"" \
"   -c <process ID>" \
"      Terminate the process whose ID is <process ID>." \
"" \
"   -h" \
"      Print this usage statement." \
   exit 1
}


# Print any number of strings padded with newlines before and after.
banner() {
   local option

   echo
   for option in "$@"; do
      echo "$option"
   done
   echo
}


# If not in autorun mode, rotate the spinner by one step.
UpdateSpinner() {
   if [ "$AUTORUN" ]; then
      return
   fi

   case "$SPINNER" in
      "|")  SPINNER="/";;
      "/")  SPINNER="-";;
      "-")  SPINNER="\\";;
      "\\") SPINNER="|";;
      *)    SPINNER="|";;
   esac

   echo -en "\rPreparing Files: $SPINNER"
}


# Check for a self contained output directory for later archiving and creates
# it if needed.
check_output_directory() {
   local dir="$1"

   if [ ! -d "$OUTPUT_DIR$dir" ]; then
      mkdir -p "$OUTPUT_DIR$dir"

      if [ $? != 0 ]; then
         banner "Could not create $OUTPUT_DIR$dir... " \
                "Have you run out of disk space?" "Continuing"
         return -1
      fi
   fi
   return 0
}


# Copy whatever files and directories you give it to
# a self contained output directory for later archiving
# Working on copies could slow this down with VERY large files but:
# 1) We don't expect VERY large files
# 2) Copying file first makes sure the archive is coherent
addArchive() {
   local file
   for file in "$@"; do
      local dir=`dirname "$file"`
      check_output_directory "$dir"
      if [ $? != 0 ]; then
         return
      fi

      # Ignore stdout and handle errors.
      UpdateSpinner
      cp -pR "$file" "$OUTPUT_DIR$dir" 2>/dev/null
   done
}

# Execute a command with:
#    stdout redirected to a file to archive.
#    stderr discarded.
runcmd() {
   local outFileRelPath="$1"
   shift # The command arguments are in "$@".

   local dir=`dirname "$outFileRelPath"`
   check_output_directory "$dir"
   if [ $? != 0 ]; then
      return
   fi

   "$@" >"$OUTPUT_DIR$outFileRelPath" 2>/dev/null
   if [ $? != 0 ]; then
      rm "$OUTPUT_DIR$outFileRelPath" 2>/dev/null
      banner "Either could not run" "$@" \
             "or could not write to $OUTPUT_DIR$outFileRelPath" \
             "Do you have a full disk?" "Continuing..."
   fi

   UpdateSpinner
}


# List (optionally recursively) the contents of a directory, with:
#    stdout redirected to a file to archive.
#    stderr discarded.
runls() {
   local dir="$1"
   local options="$2"

   if [ ! -d "$dir" ]; then
      return
   fi

   runcmd "$dir/ls.txt" ls -l"$options"@Oe -- "$dir"
}


# Find all VMware-related user or system process crash logs in the given
# directory, and add them to the archive.
add_crashlogs() {
   local file
   for file in "$1"/*.crash "$1"/*.crash.log; do
      if grep -qs '^Path:.*\(VM\|vm\)ware' "$file"; then
         addArchive "$file"
      fi
   done
}

# Cleanup our temp folder and optionally exit.
cleanup() {
   local exitCode="$1"

   rm -rf "$OUTPUT_DIR"
   if [ $? != 0 ]; then
      banner "$OUTPUT_DIR was not successfully removed." \
             "Please remove manually."
   fi

   if [ "$exitCode" ]; then
      exit "$exitCode"
   fi
}

printf -v HOME %s ~ # Was bug 841728

# Place the file on the user's desktop if possible to increase its visibility.
# Sandboxed processes can't write to the desktop so use $CFFIXED_USER_HOME instead.
if [ ! $CFFIXED_USER_HOME ]; then
   DESTDIR="$HOME/Desktop"
else
   DESTDIR="$CFFIXED_USER_HOME"
fi

ARCHIVE="$DESTDIR/$(date '+VMware Remote Console Problem Report %Y-%m-%d at %H.%M.%S').zip"

# Parse args
while [ $# -gt 0 ]; do
   case "$1" in
   -a)
      AUTORUN=1
      ;;
   -c)
      CANCEL_PID="$2"
      shift
      ;;
   -h)
      usage
      ;;
   *)
      usage
      ;;
   esac
   shift
done

# Start message

if [ "$AUTORUN" ]; then
   # Output the ID of this process (in hex).
   printf "AUTO_PROCESSID %08x\n" $$
fi

banner "Support program for VMware Remote Console - Build 21740317"

# Cancel running support tool

if [ "$CANCEL_PID" ]; then
   printf "Cancelling process %08x\n" "$CANCEL_PID"
   /bin/kill "$CANCEL_PID"
   exit # Use the status of the last command.
fi

# Create a temporary directory to place the files to archive.
#
# SECURITY (was bug 1142940):
# o mktemp creates a new directory, with a random name, and gives it
#   permissions 700 so only the current user (and root) can access the
#   contents of the directory. This prevents a rogue user from:
#   1) Guessing the name of the directory.
#   2) Performing a symlink attack inside the directory.
#   3) Reading data inside the directory, that only the current user (and root)
#      are supposed to access.
# o The directory is created inside $TMPDIR, so only the current user (and root)
#   can delete the directory. This prevents a rogue user from wholesale
#   replacing the directory after we have created it, with a directory that has
#   more lenient permissions.
OUTPUT_DIR="`mktemp -d $TMPDIR/vmrc_XXXXXX`"
if [ $? != 0 ]; then
   banner "Could not create a secure temporary directory. Exiting..."
   exit 1
fi

# Cleanup our temp folder if the process is signalled midway.
trap "cleanup 1" HUP INT QUIT TERM ABRT

# Add UI logs and file permissions - VMware Remote Console specific as well as VMware generic
addArchive "$HOME/Library/Logs/VMware Remote Console"/*.log # VMware Remote Console specific
addArchive "$HOME/Library/Logs/VMware"/*.log        # VMware generic for user
addArchive "/Library/Logs/VMware"/*.log             # VMware generic for system
runls "$HOME/Library/Logs/VMware Remote Console" R

# Logs from kernel modules no longer appear in /var/log/system.log
# or /var/log/kernel.log in macOS Sierra(10.12). Use the log command-line
# tool to view and filter log messages.
runcmd "/tmp/system.log" log show --start $(date -v-4d +"%Y-%m-%d") --style syslog

# Collect kernel panic data:
# o from Mac OS 10.6 and newer:
add_crashlogs "/Library/Logs/DiagnosticReports/"

# Collect user process crashes:
# o from Mac OS 10.6 and newer:
# More from Mac OS 10.6 and newer:
add_crashlogs "$HOME/Library/Logs/DiagnosticReports/"

# Collect local information and file permissions.
addArchive "$HOME/Library/Preferences/VMware Remote Console"
addArchive "$HOME/Library/Preferences/"com.vmware.*.plist
runls "$HOME/Library/Preferences/VMware Remote Console" R

# Terminate the spinner
echo
echo

# Perform the archiving

# SECURITY (was bug 1142940):
# umask 0077 ensures that the archive is created with permissions 600 so only
# the current user (and root) can access the contents of the archive.
(cd "$OUTPUT_DIR" && umask 0077 && zip -rv "$ARCHIVE" .)
if [ $? != 0 ]; then
   banner "The archive did not successfully complete!" \
      "You may wish to re-run this script after checking disk space, etc."
else
   if [ "$AUTORUN" ]; then
      # Output the path of the generated file.
      echo AUTO_SUCCESS "$ARCHIVE"
   fi

   st=`stat -s "$ARCHIVE"`
   size=`eval "$st" && echo "$st_size"`
   supportURL='http://www.vmware.com/support/sr/sr_login.jsp'
   if [ $size -gt 10000000 ]; then
      banner \
         "File created: $ARCHIVE" \
         "NOTE: $ARCHIVE is greater than 10 MB." \
         "Please do not attach this file when submitting an incident report." \
         "Please contact VMware support for an ftp site." \
         "To file a support incident, go to $supportURL"
   else
      banner "File created: $ARCHIVE" \
             "Please attach this file when submitting an incident report." \
             "To file a support incident, go to $supportURL"
   fi
fi

# Cleanup our temp folder on success.
trap - HUP INT QUIT TERM ABRT; cleanup
